﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceProcess;
using System.Threading;
using Microsoft.WindowsAPICodePack.Net;

namespace NetLocationMonitor
{
    public partial class NetLocationMonitor : ServiceBase
    {
        private IEnumerable<Network> _KnownNetworks;
        private Timer _Timer;

        public NetLocationMonitor()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            _KnownNetworks = NetworkListManager.GetNetworks(NetworkConnectivityLevels.All);
            _Timer = new Timer(CheckNetworks);
            _Timer.Change(TimeSpan.Zero, TimeSpan.FromMinutes(1));
        }

        protected override void OnStop()
        {
            _Timer.Dispose();
        }

        /// <summary>
        /// Checks for any new networks and sets their location to Private.
        /// </summary>
        /// <param name="args">Not used.</param>
        private void CheckNetworks(object args)
        {
            var currentNets = NetworkListManager.GetNetworks(NetworkConnectivityLevels.All);
            var diff = currentNets.Except(_KnownNetworks);
            // check all new networks that appeared
            foreach (var net in diff)
            {
                // if there is a new net that is active
                if (net.Connectivity.HasFlag(ConnectivityStates.IPv4Internet))
                {
                    // set it to private if it isn't
                    if (net.Category != NetworkCategory.Private)
                    {
                        EventLog.WriteEntry(string.Format("Setting network {0} ({1}) to private (was: {2})", net.NetworkId, net.Name, net.Category));
                        net.Category = NetworkCategory.Private;
                    }
                }
            }
        }
    }
}
