﻿using System;
using System.Linq;
using Microsoft.Win32;
using Microsoft.WindowsAPICodePack.Net;

namespace NLMConfig
{
    class Program
    {
        private const String RegKey = @"Software\NetLocationMonitor";

        static void ClearSettings()
        {
            try
            {
                var key = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
                RegistryKey subkey;
                if ((subkey = key.OpenSubKey(RegKey)) != null)
                {
                    subkey.Close();
                    key.DeleteSubKey(RegKey);
                }
            }
            catch (Exception)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(@"Failed to delete registry key {0}", RegKey);
                return;
            }
            Console.WriteLine("Settings cleared.");
        }

        static void ShowSettings()
        {
            try
            {
                var key = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
                RegistryKey subkey;
                if ((subkey = key.OpenSubKey(RegKey)) != null)
                {
                    Console.WriteLine("\nCurrently monitored connections:");
                    foreach (var name in subkey.GetValueNames())
                    {
                        String val = (String)subkey.GetValue(name);
                        var net = NetworkListManager.GetNetwork(new Guid(val));
                        Console.WriteLine("{0}: {1}", net.Name, net.NetworkId);
                    }
                    subkey.Close();
                }
                else
                {
                    Console.WriteLine("No settings found in registry.");
                }
            }
            catch (Exception)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(@"Failed to get settings");
                return;
            }
        }

        static void SaveSettings(Network[] nets, int[] nums)
        {
            RegistryKey soft = RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry64);
            RegistryKey key = soft.OpenSubKey(RegKey, true);
            if (key == null)
            {
                key = soft.CreateSubKey(RegKey);
                if (key == null)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine(@"Failed to open registry key {0} for writing, do you have privileges for it?", RegKey);
                    return;
                }
            }

            try
            {
                foreach (int i in nums)
                {
                    key.SetValue(nets[i-1].Name, nets[i-1].NetworkId.ToString());
                }
            }
            catch (Exception)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(@"Error saving settings");
                return;
            }

            Console.WriteLine("Settings saved.");
        }

        static void Main(string[] args)
        {
            Console.ForegroundColor = ConsoleColor.White;
            Console.Write("Network Location Monitor by ");
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine("Omega Red");
            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine("=====================================\n");
            Console.Write("Your network connections (");
            Console.ForegroundColor = ConsoleColor.Cyan;
            Console.Write("Name, ");
            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.Write("Location, ");
            Console.ForegroundColor = ConsoleColor.Green;
            Console.Write("Connectivity state, ");
            Console.ForegroundColor = ConsoleColor.Gray;
            Console.Write("GUID):\n");
           
            Network[] nets = NetworkListManager.GetNetworks(NetworkConnectivityLevels.All).ToArray();
            for (int i = 0; i < nets.Length; i++)
            {
                Console.ForegroundColor = ConsoleColor.White;
                Console.Write("{0}: ", i+1);
                Console.ForegroundColor = ConsoleColor.Cyan;
                Console.Write("{0} ", nets[i].Name);
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.Write("{0} ", nets[i].Category);
                Console.ForegroundColor = ConsoleColor.Green;
                Console.Write("{0} ", nets[i].Connectivity);
                Console.ForegroundColor = ConsoleColor.Gray;
                Console.Write("{0} \n", nets[i].NetworkId);
            }

            ShowSettings();

            Console.WriteLine("\nWhich connection to activate location monitoring for?\nEnter numbers separated by spaces (like 1 3 23), enter nothing to clear settings:");
            String s = Console.ReadLine();
            if (String.IsNullOrEmpty(s))
            {
                ClearSettings();
            }
            else
            {
                int[] nums;
                try
                {
                    nums = s.Trim().Split(' ', '\t', ',', ';').Select((str, idx) => int.Parse(str)).ToArray();
                }
                catch (Exception)
                {
                    Console.ForegroundColor = ConsoleColor.Red;
                    Console.WriteLine("Invalid number.");
                    return;
                }

                SaveSettings(nets, nums);
            }
            ShowSettings();
            //Console.WriteLine("Press Enter to exit.");
            //Console.ReadLine();
        }
    }
}
